/**
 * Vue Number Format 3.2.0
 * (c) 2021-2023 Dipak Sarkar <hello@dipaksarkar.in> (https://dipaksarkar.in/)
 * @license MIT
 */
import { defineComponent } from 'vue';

var options$1 = {
    prefix: '',
    suffix: '',
    separator: ',',
    decimal: '.',
    precision: 2,
    minimumFractionDigits: null,
    prefill: true,
    reverseFill: false,
    min: null,
    max: null,
    nullValue: ''
};

function between(min, n, max) {
    return Math.max(min, Math.min(n, max));
}
function fixed(precision) {
    return between(0, precision, 20);
}
function toFixed(numbers, precision) {
    const exp = Math.pow(10, precision);
    const float = parseFloat(numbers) / exp || 0;
    return float.toFixed(fixed(precision));
}
class NumberFormat {
    constructor(config) {
        this.options = Object.assign(options$1, config);
        this.input = '';
        this.number = '';
        this.isClean = true;
        const { prefix, suffix, decimal } = this.options;
        this.preSurRegExp = new RegExp(`${prefix}|${suffix}`, 'g');
        this.numberRegExp = new RegExp(`[^0-9\\${decimal}]+`, 'gi');
        this.cleanRegExp = new RegExp('[^0-9]+', 'gi');
        this.negativeRegExp = new RegExp('[^0-9\\-]+', 'gi');
    }
    isNull() {
        if (this.isClean) {
            return !this.numberOnly(this.input, this.cleanRegExp);
        }
        return !this.numberOnly(this.input, this.negativeRegExp);
    }
    clean(clean = false) {
        this.isClean = clean;
        return this;
    }
    sign() {
        if (this.input === null || this.input === undefined) {
            return '';
        }
        const hasMinus = this.input.toString().indexOf('-') >= 0;
        if (this.isClean) {
            return hasMinus && this.realNumber() > 0 ? '-' : '';
        }
        return hasMinus ? '-' : '';
    }
    toNumber(str) {
        return Number(str);
    }
    numberOnly(str, regExp) {
        return str === null || str === void 0 ? void 0 : str.toString().replace(regExp, '');
    }
    isNegative() {
        return this.sign() === '-';
    }
    numbers() {
        if (this.options.reverseFill) {
            this.number = toFixed(this.numberOnly(this.input, /\D+/g), this.options.precision).replace('.', this.options.decimal);
        }
        else if (typeof this.input === 'number') {
            this.number = this.parts(this.input.toString().replace('-', ''), '.').join(this.options.decimal);
        }
        else {
            const input = this.input.replace(this.preSurRegExp, '');
            this.number = this.numberOnly(input, this.numberRegExp);
            this.number = this.parts(this.number).join(this.options.decimal);
        }
        return this.number;
    }
    realNumber() {
        return parseFloat(this.numbers().toString().replace(this.options.decimal, '.'));
    }
    parts(num, separator) {
        const decimal = separator || this.options.decimal;
        let parts = num.toString().split(decimal);
        if (parts.length > 1) {
            parts[0] = this.toNumber(parts[0]) || 0;
            parts[1] = parts.slice(1, parts.length).join('');
            parts = parts.slice(0, 2);
        }
        if (this.isClean) {
            const newNumber = this.toNumber(parts.join('.')).toFixed(this.options.precision);
            const cleanNumber = this.toNumber(newNumber);
            const minimumDigits = cleanNumber.toFixed(this.options.minimumFractionDigits);
            const hasMinFraction = this.options.minimumFractionDigits >= 0 && cleanNumber.toString().length < minimumDigits.length;
            if (hasMinFraction) {
                parts = minimumDigits.toString().split('.');
            }
            else {
                parts = cleanNumber.toString().split('.');
            }
        }
        return parts.slice(0, 2);
    }
    addSeparator() {
        const parts = this.numbers().split(this.options.decimal);
        parts[0] = parts[0].toString().replace(/(\d)(?=(?:\d{3})+\b)/gm, `$1${this.options.separator}`);
        return parts.join(this.options.decimal);
    }
    format(input) {
        this.input = input;
        if (this.isNull() && !this.options.reverseFill) {
            return this.options.nullValue;
        }
        return this.sign() + this.options.prefix + this.addSeparator() + this.options.suffix;
    }
    unformat(input) {
        this.input = input;
        if (this.isNull() && !this.options.reverseFill) {
            return this.options.nullValue;
        }
        return this.sign() + this.realNumber();
    }
}

function cloneDeep(data) {
    return JSON.parse(JSON.stringify(data));
}
function FacadeInputEvent() {
    return new CustomEvent('input', {
        bubbles: true,
        cancelable: true,
        detail: { facade: true }
    });
}
function FacadeChangeEvent() {
    return new CustomEvent('change', {
        bubbles: true,
        cancelable: true,
        detail: { facade: true }
    });
}
function getInputElement(el) {
    const inputElement = el instanceof HTMLInputElement ? el : el.querySelector('input');
    if (!inputElement) {
        throw new Error('facade directive requires an input element');
    }
    return inputElement;
}
function updateCursor(el, position) {
    const setSelectionRange = () => {
        el.setSelectionRange(position, position);
    };
    setSelectionRange();
    setTimeout(setSelectionRange(), 1);
}
function updateValue(el, vnode, { emit = true, force = false, clean = false } = {}) {
    var _a;
    const { options, oldValue } = el;
    const currentValue = ((_a = vnode === null || vnode === void 0 ? void 0 : vnode.props) === null || _a === void 0 ? void 0 : _a.value) || el.value;
    if (force || oldValue !== currentValue) {
        const number = new NumberFormat(options).clean(clean && !options.reverseFill);
        let masked = number.format(currentValue);
        let unmasked = number.clean(options && !options.reverseFill).unformat(currentValue);
        if (clean) {
            if (options.max && Number(unmasked) > options.max) {
                masked = number.format(options.max);
                unmasked = number.unformat(options.max);
            }
            else if (options.min && Number(unmasked) < options.min) {
                masked = number.format(options.min);
                unmasked = number.unformat(options.min);
            }
        }
        el.masked = masked;
        el.unmaskedValue = unmasked;
        if (el.value !== masked) {
            el.value = masked;
        }
        return emit && el.dispatchEvent(FacadeInputEvent());
    }
}
function inputHandler(event, el) {
    const { target, detail } = event;
    if (detail === null || detail === void 0 ? void 0 : detail.facade) {
        return false;
    }
    const { oldValue, options, masked } = target;
    event.stopPropagation();
    let positionFromEnd = target.value.length;
    if (target.selectionEnd) {
        positionFromEnd = target.value.length - target.selectionEnd;
    }
    updateValue(target, null, { emit: false });
    if (options.suffix) {
        positionFromEnd = Math.max(positionFromEnd, options.suffix.length);
    }
    positionFromEnd = target.value.length - positionFromEnd;
    if (options.prefix) {
        positionFromEnd = Math.max(positionFromEnd, options.prefix.length);
    }
    updateCursor(target, positionFromEnd);
    if (oldValue !== target.value) {
        target.oldValue = masked;
        target.dispatchEvent(FacadeInputEvent());
    }
}
function blurHandler(event, el) {
    const { target, detail } = event;
    if (detail === null || detail === void 0 ? void 0 : detail.facade) {
        return false;
    }
    const { oldValue, masked } = target;
    updateValue(target, null, { force: true, emit: true, clean: true });
    if (oldValue !== target.value) {
        target.oldValue = masked;
        target.dispatchEvent(FacadeChangeEvent());
    }
}
function keydownHandler(event, el) {
    const { options } = el;
    const regExp = new RegExp(`${options.prefix}|${options.suffix}`, 'g');
    const newValue = el.value.replace(regExp, '');
    const canNegativeInput = options.min < 0 || !options.min;
    if (([110, 190].includes(event.keyCode) || event.key === options.decimal) && newValue.includes(options.decimal)) {
        event.preventDefault();
    }
    else if ([109].includes(event.keyCode) && !canNegativeInput) {
        event.preventDefault();
    }
    else if ([8].includes(event.keyCode)) {
        const selectionEnd = el.selectionEnd || 0;
        const character = el.value.slice(selectionEnd - 1, selectionEnd);
        const replace = el.value.slice(selectionEnd - 2, selectionEnd);
        if (character === options.separator) {
            event.preventDefault();
            let positionFromEnd = el.value.length - selectionEnd;
            el.value = el.value.replace(replace, '');
            if (options.suffix) {
                positionFromEnd = Math.max(positionFromEnd, options.suffix.length);
            }
            positionFromEnd = el.value.length - positionFromEnd;
            if (options.prefix) {
                positionFromEnd = Math.max(positionFromEnd, options.prefix.length);
            }
            updateCursor(el, positionFromEnd);
            el.dispatchEvent(new Event('input'));
        }
        else if ([options.prefix, '-'].includes(character)) {
            event.preventDefault();
            el.value = '';
            el.dispatchEvent(new Event('input'));
        }
    }
}

var directive = {
    beforeMount: (el, { value, modifiers }, vnode) => {
        el = getInputElement(el);
        const options = Object.assign(cloneDeep(options$1), value, modifiers);
        el.options = options;
        updateValue(el, vnode, { force: options.prefill, clean: true });
    },
    mounted: (el) => {
        el = getInputElement(el);
        const handlerOwner = el.parentElement || el;
        const oninput = (e) => {
            if (e.target !== el) {
                return;
            }
            inputHandler(e);
        };
        handlerOwner.addEventListener('input', oninput, true);
        el.onblur = (e) => blurHandler(e);
        el.onkeydown = (e) => keydownHandler(e, el);
        el.cleanup = () => handlerOwner.removeEventListener('input', oninput, true);
    },
    updated: (el, { value, oldValue, modifiers }, vnode) => {
        el = getInputElement(el);
        const options = el.options;
        el.options = Object.assign(options, value, modifiers);
        if (value !== oldValue) {
            updateValue(el, vnode, { force: true, clean: true });
        }
        else {
            updateValue(el, vnode);
        }
    },
    unmounted: (el) => {
        getInputElement(el).cleanup();
    }
};

const options = cloneDeep(options$1);
var component = defineComponent({
    name: 'VueNumber',
    directives: {
        number: directive
    },
    props: {
        modelValue: {
            type: [String, Number],
            required: true
        },
        nullValue: {
            type: [Number, String],
            default: () => options.nullValue
        },
        masked: {
            type: Boolean,
            default: false
        },
        reverseFill: {
            type: Boolean,
            default: options.reverseFill
        },
        prefill: {
            type: Boolean,
            default: options.prefill
        },
        precision: {
            type: Number,
            default: () => options.precision
        },
        minimumFractionDigits: {
            type: [Number, Boolean],
            default: () => options.minimumFractionDigits
        },
        decimal: {
            type: String,
            default: () => options.decimal
        },
        min: {
            type: [Number, Boolean],
            default: () => options.min
        },
        max: {
            type: [Number, Boolean],
            default: () => options.max
        },
        separator: {
            type: String,
            default: () => options.separator
        },
        prefix: {
            type: String,
            default: () => options.prefix
        },
        suffix: {
            type: String,
            default: () => options.suffix
        }
    },
    emits: ['update:model-value', 'input:model-value'],
    data() {
        return {
            maskedValue: this.modelValue,
            unmaskedValue: ''
        };
    },
    computed: {
        emittedValue() {
            return this.masked ? this.maskedValue : this.unmaskedValue;
        },
        config() {
            return {
                nullValue: this.nullValue,
                masked: this.masked,
                reverseFill: this.reverseFill,
                prefill: this.prefill,
                precision: this.precision,
                minimumFractionDigits: this.minimumFractionDigits,
                decimal: this.decimal,
                min: this.min,
                max: this.max,
                separator: this.separator,
                prefix: this.prefix,
                suffix: this.suffix
            };
        }
    },
    methods: {
        input(event) {
            const { target } = event;
            this.maskedValue = target.value;
            this.unmaskedValue = target.unmaskedValue;
            this.$emit('input:model-value', this.emittedValue);
        },
        change() {
            this.$emit('update:model-value', this.emittedValue);
        }
    },
    template: `<input
    v-number="config"
    type="text"
    autocomplete="off"
    :value="maskedValue"
    class="v-number vue-number-format"
    @change="change"
    @input="input"
  />`
});

const VueNumberFormat = {
    install(app, config) {
        if (config) {
            Object.assign(options$1, config);
        }
        app.directive('number', directive);
        app.component('VueNumber', component);
    }
};

export { NumberFormat, component, VueNumberFormat as default, directive, options$1 as options };
